/*
 AT Mini LED Matrix
 Version: 0.3
 Author: Alex from insideGadgets (www.insidegadgets.com)
 Created: 20/02/2013
 Last Modified: 5/05/2013
 
 */
 
#ifndef cbi
#define cbi(sfr, bit) (_SFR_BYTE(sfr) &= ~_BV(bit))
#endif
#ifndef sbi
#define sbi(sfr, bit) (_SFR_BYTE(sfr) |= _BV(bit))
#endif

#ifndef boolean
	typedef uint8_t boolean;
#endif
#ifndef bool
	typedef uint8_t bool;
#endif 
#ifndef byte
	typedef uint8_t byte;
#endif

#ifndef NULL
#define NULL ((void *)0)
#endif

#define LOW 0
#define HIGH 1
#define false 0
#define true 1

#define start_time 0
#define end_time 1
#define delay_time 2
#define start_location 4
#define end_location 3
#define animation_speed 5

// Sleep mode - power save
void system_sleep_pwr_save(void) {
	set_sleep_mode(SLEEP_MODE_PWR_SAVE); // Set sleep mode
	sleep_mode(); // System sleeps here
}

// Sleep mode - power down
void system_sleep(void) {
	set_sleep_mode(SLEEP_MODE_PWR_DOWN); // Set sleep mode
	sleep_mode(); // System sleeps here
}

// Setup the ATmega
void setup(void) {
	// Set Outputs
	DDRB |= (1<<PB5) | (1<<PB4) | (1<<PB3) | (1<<PB2) | (1<<PB0);
	DDRC |= (1<<PC3) | (1<<PC2) | (1<<PC1) | (1<<PC0);
	DDRD |= (1<<PD7) | (1<<PD6) | (1<<PD5) | (1<<PD4) |  (1<<PD3) | (1<<PD2) | (1<<PD1);
	
	// Turn on pull-up resistors on other ports to save power
	PORTB |= (1<<PB1);
	PORTC |= (1<<PC5) | (1<<PC4);
	
	// Enable the 32.768KHz crystal as a RTC
	TIMSK2 = 0; // Disable the Timer/Counter2 interrupts by clearing OCIE2x and TOIE2
	ASSR = (1<<AS2); // Select clock source by setting AS2 as appropriate
	TCNT2 = 0;// Write new values to TCNT2, OCR2x, and TCCR2x
	OCR2A = 0;
	OCR2B = 0;
	TCCR2A = 0;
	TCCR2B = (1<<CS22) | (1<<CS21) | (1<<CS20); // 1024 prescaler for 8 seconds overflow on 32.768KHz crystal
	//TCCR2B = (1<<CS22) | (1<<CS20); // 128 prescaler for 1 second overflow on 32.768KHz crystal
	while ((ASSR & ((1<<TCN2UB) | (1<<OCR2AUB) | (1<<OCR2BUB) | (1<<TCR2AUB) | (1<<TCR2BUB))) != 0); // To switch to asynchronous operation: Wait for TCN2xUB, OCR2xUB, and TCR2xUB ASSR
	TIFR2 = 0; // Clear the Timer/Counter2 Interrupt Flags
	TIMSK2 |= (1<<TOIE2); // Enable overflow interrupt
	
	// After a Power-up Reset or wake-up from Power-down or Standby mode, the user should be aware of the fact 
	// that this Oscillator might take as long as one second to stabilize
	_delay_ms(1000);
	
	sei(); // Turn on interrupts
}
